import { createLibp2p } from "libp2p";
import { DirectSub } from "@dao-xyz/libp2p-direct-sub";
import { DirectBlock, MemoryLevelBlockStore, LevelBlockStore, } from "@dao-xyz/libp2p-direct-block";
import { noise } from "@dao-xyz/libp2p-noise";
import { mplex } from "@libp2p/mplex";
import { Level } from "level";
import { transports, relay } from "./transports.js";
const isNode = typeof window === undefined || typeof window === "undefined";
export const createLibp2pExtended = async (args) => {
    let peer;
    if (args?.libp2p?.start) {
        peer = args?.libp2p;
    }
    else {
        const opts = args?.libp2p;
        peer = (await createLibp2p({
            peerId: opts?.peerId,
            connectionManager: opts?.connectionManager || {
                minConnections: 0,
            },
            addresses: opts?.addresses || {
                listen: ["/ip4/127.0.0.1/tcp/0", "/ip4/127.0.0.1/tcp/0/ws"],
            },
            transports: opts?.transports || transports(),
            relay: relay(),
            connectionEncryption: [noise()],
            streamMuxers: [mplex()],
        }));
    }
    if (!peer.directsub) {
        peer.directsub = new DirectSub(peer, {
            canRelayMessage: true,
            signaturePolicy: "StrictNoSign",
            connectionManager: args?.pubsub,
        });
    }
    else if (args?.pubsub) {
        throw new Error("Directsub already seet on client, but 'pubsub' constructor arguments are provided which is unexpected");
    }
    peer.directblock =
        peer.directblock ||
            new DirectBlock(peer, args?.blocks?.directory
                ? new LevelBlockStore(new Level(args.blocks.directory))
                : new MemoryLevelBlockStore());
    const start = peer.start.bind(peer);
    peer.start = async () => {
        if (!peer.isStarted()) {
            await start();
        }
        await Promise.all([peer.directblock.start(), peer.directsub.start()]);
    };
    const stop = peer.stop.bind(peer);
    peer.stop = async () => {
        await stop();
        await Promise.all([peer.directblock.stop(), peer.directsub.stop()]);
    };
    return peer;
};
//# sourceMappingURL=index.js.map